// Configure the framework
#include "SoulissFramework.h"
#include "bconf/StandardArduino.h" // Use a standard Arduino
#include "conf/ethW5100.h"                  // Ethernet through Wiznet W5100
//#include "conf/Gateway.h" // The main node is the Gateway, we have just one node


// Include framework code and libraries
#include <SPI.h>      // RC522 Module uses SPI protocol
#include <MFRC522.h>   // Library for Mifare RC522 Devices
#include <EEPROM.h>  // We are going to read and write PICC's UIDs from/to EEPROM
#include "Souliss.h"

// Define the network configuration according to your router settings
uint8_t ip_address[4]  = {192, 168, 27, 79};
uint8_t subnet_mask[4] = {255, 255, 255, 0};
uint8_t ip_gateway[4]  = {192, 168, 27, 1};

#define myvNet_address ip_address[3] // The last byte of the IP address (77) is also the vNet address
#define Gateway_address 77
#define Peer_address    78
#define Peer_address    79


#define Alarm_ARMED 0 // This identify the number of the logic on this node
#define Alarm_ACTIVE 1 // This identify the number of the logic on this node
#define MotionSensor1 2
#define MotionSensor2 3
#define PIR_power 4
#define Alarm_Toggle 5

/* We need to define MFRC522's pins and create instance
   Pin layout should be as follows (on Arduino Uno):
   MOSI: Pin 11 / ICSP-4
   MISO: Pin 12 / ICSP-1
   SCK : Pin 13 / ICSP-3
   SS : Pin 4 (Configurable)
   RST : Pin 5 (Configurable)

   look MFRC522 Library for
   pin configuration for other Arduinos.
*/


#define SS_PIN 4
#define RST_PIN 5

MFRC522 mfrc522(SS_PIN, RST_PIN); // Create MFRC522 instance.
boolean match = false; // initialize card match to false
boolean programMode = false; // initialize programming mode to false

int successRead; // Variable integer to keep if we have Successful Read from Reader
byte storedCard[4];   // Stores an ID read from EEPROM
byte readCard[4];           // Stores scanned ID read from RFID Module
byte masterCard[4]; // Stores master card's ID read from EEPROM

#define COMMON_ANODE
#ifdef COMMON_ANODE
#define LED_ON LOW
#define LED_OFF HIGH
#else
#define LED_ON HIGH
#define LED_OFF LOW
#endif

#define beeper A0
#define led 10
#define Alarm_ArmedLED A1
#define Alarm_ActiveLED A2

#define wipeB 2 // Button pin for WipeMode connect to 5V to Wipe data, connect to GND for normal use.
#define redLed   led //7  
#define greenLed led //6
#define blueLed  led //5

#define PIR1_1_on     0xFF01,0x01
#define PIR1_1_off    0xFF01,0x02
#define PIR1_2_on     0xFF02,0x01
#define PIR1_2_off    0xFF02,0x02
#define Alarm_on      0xFF03,0x01
#define Alarm_off     0xFF03,0x02
#define Alarm_sirenON   0xFF04,0x01
#define Alarm_sirenOFF   0xFF04,0x02
#define PIRpower_on   0xFF05, 0x01
#define PIRpower_off   0xFF05, 0x02


int ledState = LOW;

void setup()
{
  Serial.begin(57600);
  SPI.begin();           // MFRC522 Hardware uses SPI protocol
  mfrc522.PCD_Init();    // Initialize MFRC522 Hardware
  mfrc522.PCD_SetAntennaGain(mfrc522.RxGain_max); //Set Antenna Gain to Max- this will increase reading distance
  Initialize();

  // Set network parameters
  Souliss_SetIPAddress(ip_address, subnet_mask, ip_gateway);

  // SetAsGateway(myvNet_address); // Set this node as gateway for SoulissApp
  Set_T11(Alarm_ARMED); // Define a simple LED light logic
  Set_T11(Alarm_ACTIVE);
  Set_T11(PIR_power);
  Set_T11(MotionSensor1);
  Set_T11(MotionSensor2);
  Set_T11(Alarm_Toggle);


  pinMode(led, OUTPUT); // Power the LED
  pinMode(beeper, OUTPUT); // Power the LED
  pinMode(Alarm_ArmedLED, OUTPUT); // Power the LED
  pinMode(Alarm_ActiveLED, OUTPUT); // Power the LED

 // Serial.flush();

mInput(MotionSensor1) = 0x04;
mInput(MotionSensor2) = 0x04;

  Wipe();
}




void loop()
{
  //Serial.println(freeMemory());


  // Here we start to play
  EXECUTEFAST() {
    UPDATEFAST();

    FAST_210ms() { // We process the logic and relevant input and output every 50 milliseconds


      //Serial.println(readRFID(Alarm_ARMED, Souliss_T1n_ToggleCmd)); //DEBUG 0-NONE 1-OK 2-KO 3-PROGRAM MODE 4-ADDING 5-REMOVINH 6-EXIT PROGRAM MODE
      readRFID(Alarm_Toggle, Souliss_T1n_ToggleCmd);
    }
FAST_510ms() {    

      Logic_T11(Alarm_ARMED); // Drive the relay coil as per command
      Logic_T11(Alarm_ACTIVE);

      DigOut(Alarm_ArmedLED, Souliss_T1n_Coil, Alarm_ARMED); // Use the pin9 to give power to the coil according to the logic
      DigOut(Alarm_ActiveLED, Souliss_T1n_Coil, Alarm_ACTIVE);


   //   Serial.println(mOutput(Alarm_ACTIVE));


    }
    FAST_510ms() {
      
      if(sbscrb(PIR1_1_on)){
        mInput(MotionSensor1) = Souliss_T1n_OnCmd;
        Serial.println(mOutput(MotionSensor1));
        Logic_T11(MotionSensor1);
    }
    
    if(sbscrb(PIR1_1_off)) {
      mInput(MotionSensor1) = Souliss_T1n_OffCmd;
      Serial.println(mOutput(MotionSensor1));
      Logic_T11(MotionSensor1);
    }

    if(sbscrb(PIR1_2_on)){
      mInput(MotionSensor2) = Souliss_T1n_OnCmd;
      Serial.println(mOutput(MotionSensor2));
      Logic_T11(MotionSensor2);
    }
    
    if(sbscrb(PIR1_2_off)) {
      mInput(MotionSensor2) = Souliss_T1n_OffCmd;
      Serial.println(mOutput(MotionSensor2));
      Logic_T11(MotionSensor2);
    }
    }

    // Process data communication
    FAST_PeerComms();
    START_PeerJoin();
  }

  EXECUTESLOW() {
    UPDATESLOW();
    SLOW_10s() { // We handle the light timer with a 10 seconds base time
      Timer_T11(Alarm_Toggle);
    }
  }
}
///////////////////////////////////////// Main Loop ///////////////////////////////////
byte readRFID(uint8_t slot, uint8_t command) {
  if (getID() == 1) {
    if (programMode) {

      if ( isMaster(readCard) ) {  //If master card scanned again exit program mode

        Serial.println("This is Master");
        Serial.println("Exiting");
        Serial.println("-----------------------------");
        programMode = false;
        return 6;

      }
      else {
        if ( findID(readCard) ) { //If scanned card is known delete it

          Serial.println("Removing");
          deleteID(readCard);
          return 5;
          //Serial.println("-----------------------------");

        }
        else {                    // If scanned card is not known add it

         Serial.println("Adding...");
          writeID(readCard);
          return 4;
          Serial.println("-----------------------------");

        }
      }
    }
    else {
      if ( isMaster(readCard) ) {  // If scanned card's ID matches Master Card's ID enter program mode
        programMode = true;

       Serial.println("Program Mode");
        byte count = EEPROM.read(0); // Read the first Byte of EEPROM that
        return 3;
              Serial.print("I have ");    // stores the number of ID's in EEPROM
             Serial.print(count);
              Serial.print(" record(s) on EEPROM");
             Serial.println("");
             Serial.println("Scan a PICC to ADD or REMOVE");
        Serial.println("-----------------------------");

      }
      else {
        if ( findID(readCard) ) {        // If not, see if the card is in the EEPROM

          Serial.println("OK"); //"Welcome, You shall pass"

          if (mOutput(Alarm_ARMED) == 1) {
            //            ledState = LOW;
            //            digitalWrite(PIR_pin, LOW);
            digitalWrite(beeper, HIGH);
            delay(700);
            digitalWrite(beeper, LOW);
          //  digitalWrite(led, LOW);
            digitalWrite(Alarm_ActiveLED, LOW);
            
            mInput(Alarm_ARMED) = Souliss_T1n_OffCmd;
            mInput(Alarm_ACTIVE) = Souliss_T1n_OffCmd;
            mInput(PIR_power) = Souliss_T1n_OffCmd;
            pblsh(Alarm_off);
            pblsh(Alarm_sirenOFF);
            pblsh(PIRpower_off);
           
         
            mInput(slot) = command;
            return 1;
            //          backupMillis = 0;

          }

        

            
          if (mOutput(Alarm_ARMED) == 0) {
            mInput(PIR_power) = Souliss_T1n_OnCmd;
            pblsh(PIRpower_on);
          
           
            Serial.println( findID(readCard) );
            //            digitalWrite(PIR_pin, HIGH);
            for (int i = 0; i <= 5; i++) {
              digitalWrite(beeper, HIGH);
              digitalWrite(Alarm_ArmedLED, HIGH);
              delay(200);
              digitalWrite(beeper, LOW);
              digitalWrite(Alarm_ArmedLED, LOW);
              delay(1500);
              if (i == 5) {
                digitalWrite(beeper, HIGH);
                digitalWrite(Alarm_ArmedLED, HIGH);
                delay(500);
                digitalWrite(beeper, LOW);
                digitalWrite(Alarm_ArmedLED, LOW);
                delay(500);
                digitalWrite(beeper, HIGH);
                digitalWrite(Alarm_ArmedLED, HIGH);
                delay(500);
                digitalWrite(beeper, LOW);
                digitalWrite(Alarm_ArmedLED, LOW);
                delay(500);
                digitalWrite(Alarm_ArmedLED, HIGH);
                mInput(Alarm_ARMED) = Souliss_T1n_OnCmd;
                pblsh(Alarm_on);
                
                
                mInput(slot) = command;
                return 1;
              }
              if  (getID() == 1 && findID(readCard)) {
                mInput(PIR_power) = Souliss_T1n_OffCmd;
            pblsh(PIRpower_off);
            pblsh(Alarm_sirenOFF);
            pblsh(Alarm_off);
               // // // Send(0xD001, 1, mInput(PIR_power));
                digitalWrite(beeper, HIGH);
                delay(700);
                digitalWrite(beeper, LOW);
                digitalWrite(Alarm_ArmedLED, LOW);
                break;
              }
            }
 



          }

      

        }


        else {    // If not, show that the ID was not valid

          Serial.println("KO"); //"You shall not pass"
          //failed();
          return 2;
        }
      }
    }
  }
  //END if (getID()==1){
  if  (mOutput(Alarm_ARMED) == 1 && mOutput(Alarm_ACTIVE) == 0 && ( mOutput(MotionSensor1) == 1 || mOutput(MotionSensor2) == 1))
  {
    for (int i = 0; i <= 5; i++) {
      digitalWrite(beeper, HIGH);
      digitalWrite(Alarm_ActiveLED, HIGH);
      delay(200);
      digitalWrite(beeper, LOW);
      digitalWrite(Alarm_ActiveLED, LOW);
      delay(1500);
      if (i == 5) {
        digitalWrite(beeper, HIGH);
        digitalWrite(Alarm_ActiveLED, HIGH);
        delay(500);
        digitalWrite(beeper, LOW);
        digitalWrite(Alarm_ActiveLED, LOW);
        delay(500);
        digitalWrite(beeper, HIGH);
        digitalWrite(Alarm_ActiveLED, HIGH);
        delay(500);
        digitalWrite(beeper, LOW);
        digitalWrite(Alarm_ActiveLED, LOW);
        delay(500);
        //  mInput(slot) = command;
        // return 1;
        mInput(Alarm_ACTIVE) = Souliss_T1n_OnCmd;
        pblsh(Alarm_sirenON);
      
        digitalWrite(Alarm_ActiveLED, HIGH);
      }
      if  (getID() == 1 && findID(readCard)) {
        digitalWrite(beeper, HIGH);
        digitalWrite(Alarm_ActiveLED, HIGH);
        delay(700);
        digitalWrite(beeper, LOW);
        digitalWrite(Alarm_ActiveLED, LOW);
        mInput(PIR_power) = Souliss_T1n_OffCmd;
        mInput(Alarm_ARMED) = Souliss_T1n_OffCmd;
        mInput(Alarm_ACTIVE) = Souliss_T1n_OffCmd;

        pblsh(Alarm_sirenOFF);
        pblsh(PIRpower_off);
        pblsh(Alarm_off);
        digitalWrite(Alarm_ActiveLED, LOW);
        mInput(slot) = command;
        return 1;
        break;
      }
    }
  }
  return 0;
}
///////////////////////////////////////// Get PICC's UID ///////////////////////////////////
int getID() {
  // Getting ready for Reading PICCs
  if ( ! mfrc522.PICC_IsNewCardPresent()) { //If a new PICC placed to RFID reader continue
    return 0;
  }
  if ( ! mfrc522.PICC_ReadCardSerial()) { //Since a PICC placed get Serial and continue
    return 0;
  }
  // There are Mifare PICCs which have 4 byte or 7 byte UID care if you use 7 byte PICC
  // I think we should assume every PICC as they have 4 byte UID
  // Until we support 7 byte PICCs

//  Serial.println("Scanned:");

  for (int i = 0; i < 4; i++) {  //
    readCard[i] = mfrc522.uid.uidByte[i];
//    Serial.print(readCard[i], HEX);
  }
//  Serial.println("");
  mfrc522.PICC_HaltA(); // Stop reading
  return 1;
}
//////////////////////////////////////// Read an ID from EEPROM //////////////////////////////
void readID( int number ) {
  int start = (number * 4 ) + 2; // Figure out starting position
  for ( int i = 0; i < 4; i++ ) { // Loop 4 times to get the 4 Bytes
    storedCard[i] = EEPROM.read(start + i); // Assign values read from EEPROM to array
  }
}
///////////////////////////////////////// Check Bytes   ///////////////////////////////////
boolean checkTwo ( byte a[], byte b[] ) {
  if ( a[0] != NULL ) // Make sure there is something in the array first
    match = true; // Assume they match at first
  for ( int k = 0; k < 4; k++ ) { // Loop 4 times
    if ( a[k] != b[k] ) // IF a != b then set match = false, one fails, all fail
      match = false;
  }
  if ( match ) { // Check to see if if match is still true
    return true; // Return true
  }
  else  {
    return false; // Return false
  }
}
////////////////////// Check readCard IF is masterCard   ///////////////////////////////////
// Check to see if the ID passed is the master programing card
boolean isMaster( byte test[] ) {
  if ( checkTwo( test, masterCard ) )
    return true;
  else
    return false;
}
///////////////////////////////////////// Find ID From EEPROM   ///////////////////////////////////
boolean findID( byte find[] ) {

  byte count = EEPROM.read(0); // Read the first Byte of EEPROM that

  for ( int i = 1; i <= count; i++ ) {  // Loop once for each EEPROM entry
    readID(i); // Read an ID from EEPROM, it is stored in storedCard[4]
    if ( checkTwo( find, storedCard ) ) { // Check to see if the storedCard read from EEPROM
      return true;
      break; // Stop looking we found it
    }
    else {  // If not, return false
    }
  }
  return false;
}

///////////////////////////////////////// Find Slot   ///////////////////////////////////
int findIDSLOT( byte find[] ) {

  byte count = EEPROM.read(0); // Read the first Byte of EEPROM that

  for ( int i = 1; i <= count; i++ ) { // Loop once for each EEPROM entry
    readID(i); // Read an ID from EEPROM, it is stored in storedCard[4]
    if ( checkTwo( find, storedCard ) ) { // Check to see if the storedCard read from EEPROM
      // is the same as the find[] ID card passed
      return i; // The slot number of the card
      break; // Stop looking we found it
    }
  }
}
///////////////////////////////////////// Add ID to EEPROM   ///////////////////////////////////
void writeID( byte a[] ) {
  if ( !findID( a ) ) { // Before we write to the EEPROM, check to see if we have seen this card before!

    byte num = EEPROM.read(0); // Get the numer of used spaces, position 0 stores the number of ID cards

    int start = ( num * 4 ) + 6; // Figure out where the next slot starts
    num++; // Increment the counter by one
    EEPROM.write( 0, num ); // Write the new count to the counter
    for ( int j = 0; j < 4; j++ ) { // Loop 4 times
      EEPROM.write( start + j, a[j] ); // Write the array values to EEPROM in the right position
    }

    //successWrite();
  }
  else {
    //failedWrite();

  }
}
///////////////////////////////////////// Remove ID from EEPROM   ///////////////////////////////////
void deleteID( byte a[] ) {
  if ( !findID( a ) ) { // Before we delete from the EEPROM, check to see if we have this card!

    //failedWrite(); // If not
  }
  else {
    byte num = EEPROM.read(0); // Get the numer of used spaces, position 0 stores the number of ID cards

    int slot; // Figure out the slot number of the card
    int start;// = ( num * 4 ) + 6; // Figure out where the next slot starts
    int looping; // The number of times the loop repeats
    int j;

    byte count = EEPROM.read(0); // Read the first Byte of EEPROM that stores number of cards

    slot = findIDSLOT( a ); //Figure out the slot number of the card to delete
    start = (slot * 4) + 2;
    looping = ((num - slot) * 4);
    num--; // Decrement the counter by one
    EEPROM.write( 0, num ); // Write the new count to the counter
    for ( j = 0; j < looping; j++ ) { // Loop the card shift times
      EEPROM.write( start + j, EEPROM.read(start + 4 + j)); // Shift the array values to 4 places earlier in the EEPROM
    }
    for ( int k = 0; k < 4; k++ ) { //Shifting loop
      EEPROM.write( start + j + k, 0);
    }

    //successDelete();
  }
}
/*

  ///////////////////////////////////////// Write Success to EEPROM   ///////////////////////////////////
  // Flashes the green LED 3 times to indicate a successful write to EEPROM
  void successWrite() {
  Serial.println("Added");

  }
  ///////////////////////////////////////// Write Failed to EEPROM   ///////////////////////////////////
  // Flashes the red LED 3 times to indicate a failed write to EEPROM
  void failedWrite() {
  Serial.println("Failed");

  }
  ///////////////////////////////////////// Success Remove UID From EEPROM  ///////////////////////////////////
  // Flashes the blue LED 3 times to indicate a success delete to EEPROM
  void successDelete() {
  Serial.println("Removed");
  }
*/
////////////////////// Wipe Function   ///////////////////////////////////

//Wipe Code if Button Pressed while setup run (powered on) it wipes EEPROM
void Wipe() {
  pinMode(wipeB, INPUT);
  //Serial.println(digitalRead(wipeB));
  if (digitalRead(wipeB) == HIGH) {
    digitalWrite(redLed, LED_ON);   // Red Led stays on to inform user we are going to wipe
    //Serial.println("Wipe Button Pressed");
    //Serial.println("You have 5 seconds to Cancel");
    //Serial.println("This will be remove all records and cannot be undone");

    delay(5000);    // Give user enough time to cancel operation

    if (digitalRead(wipeB) == HIGH) {  // If button still be pressed, wipe EEPROM
      //Serial.println("Starting Wiping EEPROM");

      for (int x = 0; x < 1024; x = x + 1) { //Loop end of EEPROM address
        if (EEPROM.read(x) == 0) { //If EEPROM address 0
          // do nothing, already clear, go to the next address in order to save time and reduce writes to EEPROM
        }
        else {
          EEPROM.write(x, 0); // if not write 0, it takes 3.3mS
        }
      }

      //Serial.println("Wiped");

      digitalWrite(redLed, LED_OFF); // visualize successful wipe

      delay(100);
      digitalWrite(redLed, LED_ON);
      delay(100);
      digitalWrite(redLed, LED_OFF);
      delay(100);
      digitalWrite(redLed, LED_ON);
      delay(100);
      digitalWrite(redLed, LED_OFF);
    }
    else {
      //Serial.println("Wiping Cancelled");

      digitalWrite(redLed, LED_OFF);
    }
  }
  //Check if master card defined, if not let user choose a master card
  //This also useful to just redefine Master Card
  //You can keep other EEPROM records just write other than 1 to EEPROM address 1

  if (EEPROM.read(1) != 143) {  // Look EEPROM if Master Card defined, EEPROM address 1 holds if defined
    // 143 our magical number
    //Serial.println("Scan A Master PICC");
    //Serial.println("Scan A PICC to Define as Master Card");

    do {
      successRead = getID(); // sets successRead to 1 when we get read from reader otherwise 0

      digitalWrite(blueLed, LED_ON); // Visualize Master Card need to be defined

      delay(1000);
      digitalWrite(blueLed, LED_OFF);
      delay(1000);

    }
    while (!successRead); //the program will not go further while you not get a successful read
    for ( int j = 0; j < 4; j++ ) { // Loop 4 times
      EEPROM.write( 2 + j, readCard[j] ); // Write scanned PICC's UID to EEPROM, start from address 3
    }

    EEPROM.write(1, 143); //Write to EEPROM we defined Master Card.
    //Serial.println("Master Card Defined");
  }
  // Serial.println("##### RFID Door Acces Control v2.0.8 #####"); //For debug purposes
  //Serial.println("Master Card's UID");

  for ( int i = 0; i < 4; i++ ) {     // Read Master Card's UID from EEPROM
    masterCard[i] = EEPROM.read(2 + i); // Write it to masterCard

    //Serial.print(masterCard[i], HEX);
  }
  //Serial.println("");
  //Serial.println("Waiting PICCs to bo scanned :)");
}
